@@ -11,7 +11,8 @@ class AssistantBase(object):
     name = 'base'
     fullname = 'Base'
     description = ''
-    role = 'crt'
+    superassistant = None
+    role = settings.DEFAULT_ASSISTANT_ROLE
     args = []
     source_file = ''
 
