@@ -12,7 +12,7 @@ class ExecutableAssistant(assistant_base.AssistantBase):
 
 class CreatorAssistant(ExecutableAssistant):
     def get_subassistants(self):
-        sa = yaml_assistant_loader.YamlAssistantLoader.get_top_level_assistants(roles=['crt'])
+        sa = yaml_assistant_loader.YamlAssistantLoader.get_assistants(superassistants=[self])
         return sa
 
     name = 'crt'
@@ -21,7 +21,7 @@ class CreatorAssistant(ExecutableAssistant):
 
 class ModifierAssistant(ExecutableAssistant):
     def get_subassistants(self):
-        sa = yaml_assistant_loader.YamlAssistantLoader.get_top_level_assistants(roles=['mod'])
+        sa = yaml_assistant_loader.YamlAssistantLoader.get_assistants(superassistants=[self])
         return sa
 
     name = 'mod'
@@ -30,17 +30,27 @@ class ModifierAssistant(ExecutableAssistant):
 
 class PreparerAssistant(ExecutableAssistant):
     def get_subassistants(self):
-        sa = yaml_assistant_loader.YamlAssistantLoader.get_top_level_assistants(roles=['prep'])
+        sa = yaml_assistant_loader.YamlAssistantLoader.get_assistants(superassistants=[self])
         return sa
 
     name = 'prep'
     fullname = 'Prepare Environment'
-    description = 'Prepare environment for upstream projects or various tasks with DevAssistant.'
+    description = 'Prepare environment for upstream projects with DevAssistant.'
+
+class TaskAssistant(ExecutableAssistant):
+    def get_subassistants(self):
+        sa = yaml_assistant_loader.YamlAssistantLoader.get_assistants(superassistants=[self])
+        return sa
+
+    name = 'task'
+    fullname = 'Custom Task'
+    description = 'Perform a custom task not related to a specific project with DevAssistant.'
 
 class TopAssistant(assistant_base.AssistantBase):
     _assistants = []
     def get_subassistants(self):
         # cache assistants to always return the same instances
         if not self._assistants:
-            self._assistants = [CreatorAssistant(), ModifierAssistant(), PreparerAssistant()]
+            self._assistants = [CreatorAssistant(), ModifierAssistant(),
+                                PreparerAssistant(), TaskAssistant()]
         return self._assistants
