@@ -2,13 +2,13 @@ import logging
 import os
 import sys
 
-from devassistant.actions import actions
+from devassistant import actions
 from devassistant import bin
 from devassistant.cli import argparse_generator
+from devassistant import current_run
 from devassistant import exceptions
 from devassistant import logger
 from devassistant import path_runner
-from devassistant import settings
 
 class CliRunner(object):
     cur_handler = None
@@ -35,18 +35,20 @@ class CliRunner(object):
         3. Parses args and decides what to run
         4. Runs a proper assistant or action
         """
+        # set current_run.USE_CACHE before constructing parser, since constructing
+        # parser requires loaded assistants
+        current_run.USE_CACHE = False if '--no-cache' in sys.argv else True
         cls.register_console_logging_handler(logger.logger)
         cls.inform_of_short_bin_name(sys.argv[0])
         top_assistant = bin.TopAssistant()
         tree = top_assistant.get_subassistant_tree()
         argparser = argparse_generator.ArgparseGenerator.\
-                        generate_argument_parser(tree, actions=actions)
+                        generate_argument_parser(tree, actions=actions.actions)
         parsed_args = argparser.parse_args()
         if parsed_args.da_debug:
             cls.change_logging_level(logging.DEBUG)
-        first_subparser = vars(parsed_args)[settings.SUBASSISTANT_N_STRING.format(0)]
-        if first_subparser in actions:
-            to_run = actions[first_subparser]
+        if actions.is_action_run(**vars(parsed_args)):
+            to_run = actions.get_action_to_run(**vars(parsed_args))
         else:
             path = top_assistant.get_selected_subassistant_path(**vars(parsed_args))
             to_run = path_runner.PathRunner(path, vars(parsed_args))
