@@ -3,6 +3,7 @@ import argparse
 from gettext import gettext as _
 
 from devassistant import settings
+from devassistant.actions import HelpAction
 
 class ArgumentParser(argparse.ArgumentParser):
     def __init__(self, *args, **kwargs):
@@ -10,13 +11,24 @@ class ArgumentParser(argparse.ArgumentParser):
 
     def error(self, message):
         import sys as _sys
-        self.print_usage(_sys.stderr)
-        # python 2 and 3 hooks to grab missing subassistant argument and report it meaningfully
-        if message == _('too few arguments') or settings.SUBASSISTANT_N_STRING[:-3] in message:
-            for action in self._get_positional_actions():
-                if isinstance(action, argparse._SubParsersAction):
-                    self.exit(2, _('You have to select a subassistant.\n'))
-        self.exit(1, _('%s: error: %s\n') % (self.prog, message))
+        # dirty hack - only top level binary has suppressed usage
+        if self.usage == argparse.SUPPRESS:
+            wants_help = '-h' in _sys.argv or '--help' in _sys.argv
+            if not wants_help:
+                print('Couldn\'t parse input, displaying help ...\n')
+            print(HelpAction.get_help())
+            if wants_help:
+                self.exit(0)
+            else:
+                self.exit(1)
+        else:
+            self.print_usage(_sys.stderr)
+            # python 2 and 3 hooks to grab missing subassistant argument and report it meaningfully
+            if message == _('too few arguments') or settings.SUBASSISTANT_N_STRING[:-3] in message:
+                for action in self._get_positional_actions():
+                    if isinstance(action, argparse._SubParsersAction):
+                        self.exit(2, _('You have to select a subassistant.\n'))
+            self.exit(1, _('%s: error: %s\n') % (self.prog, message))
 
 class DefaultIffUsedActionFactory(object):
     """Argparse doesn't cover one usecase of default values:
