@@ -7,13 +7,26 @@ import signal
 import subprocess
 import sys
 
+from devassistant import current_run
 from devassistant import exceptions
 from devassistant.logger import logger
 
 class ClHelper(object):
     @classmethod
-    def run_command(cls, cmd_str, log_level=logging.DEBUG, scls=[], ignore_sigint=False):
-        """Runs a command from string, e.g. "cp foo bar" """
+    def run_command(cls,
+                    cmd_str,
+                    log_level=logging.DEBUG,
+                    scls=[],
+                    ignore_sigint=False,
+                    output_callback=None):
+        """Runs a command from string, e.g. "cp foo bar"
+        Args:
+            cmd_str: the command to run as string
+            log_level: level at which to log command output (DEBUG by default)
+            scls: list of ['enable', 'foo', 'bar'] (scriptlet name + arbitrary number of scl names)
+            ignore_sigint: should we ignore sigint during this command (False by default)
+            output_callback: function that gets called with every line of output as argument
+        """
         # format for scl execution if needed
         cmd_str = cls.format_for_scls(cmd_str, scls)
         logger.log(log_level, cmd_str, extra={'event_type': 'cmd_call'})
@@ -40,29 +53,33 @@ class ClHelper(object):
                                 preexec_fn=preexec_fn)
         stdout = []
         while proc.poll() == None:
-            output = proc.stdout.readline().strip().decode('utf8')
-            stdout.append(output)
+            output = proc.stdout.readline().decode('utf8')
             if output:
+                output = output.strip()
+                stdout.append(output)
                 logger.log(log_level, output, extra={'event_type': 'cmd_out'})
+            if output_callback:
+                output_callback(output)
         stdout = '\n'.join(stdout)
         # there may be some remains not read after exiting the previous loop
         output_rest = proc.stdout.read().strip().decode('utf8')
         # we want to log lines separately, not as one big chunk
         output_rest_lines = output_rest.splitlines()
-        for i, l in enumerate(output_rest.splitlines()):
+        for i, l in enumerate(output_rest_lines):
             logger.log(log_level, l, extra={'event_type': 'cmd_out'})
-            # if rest of the output doesn't end with \n, we don't add \n after last line
-            if i == len(output_rest_lines) - 1 and not output_rest.endswith('\n'):
-                stdout += l
-            # else we add it everytime
-            else:
-                stdout += l + '\n'
+            # add newline for every line - for last line, only add it if it was originally present
+            if i != len(output_rest_lines) - 1 or output_rest.endswith('\n'):
+                l += '\n'
+            stdout += l
+            if output_callback:
+                output_callback(l)
 
         # log return code always on debug level
         logger.log(logging.DEBUG, proc.returncode, extra={'event_type': 'cmd_retcode'})
+        stdout = stdout.strip()
 
         if proc.returncode == 0:
-            return stdout.strip()
+            return stdout
         else:
             raise exceptions.ClException(cmd_str,
                                          proc.returncode,
@@ -111,8 +128,6 @@ class DialogHelper(object):
     Zenity, possibly other registered).
     """
     helpers = {}
-    # can be set to something different from the respective UI frontend
-    use_helper = 'cli'
 
     @classmethod
     def register_helper(cls, helper):
@@ -122,7 +137,7 @@ class DialogHelper(object):
 
     @classmethod
     def get_appropriate_helper(cls):
-        return cls.helpers[cls.use_helper]
+        return cls.helpers[current_run.UI]
 
     @classmethod
     def ask_for_password(cls, prompt='Provide your password:', **options):
