@@ -1,43 +1,54 @@
 # -*- coding: utf-8 -*-
+"""General notes on dependency installation:
+The only class that should be used from outside is DependencyInstaller. It uses PackageManager
+subclasses to install dependencies collected from assistants/snippets, e.g.
+[{'rpm': ['package1', 'package2'], 'pip': ['spam']}, {'gem': ['foo']}]
 
+DevAssistant distinguishes two general types of dependencies:
+- system dependencies (rpm, pacman, ...)
+- non-system dependencies
+
+System dependencies are special, since DependencyInstaller throws away all system
+dependencies that are not native to this system. E.g. it throws away RPM deps on
+ArchLinux. Non-system dependencies are always processed, since they should be
+distro-agnostic.
+
+PackageManager subclasses represent the tools that are used to install the dependencies.
+Note, that sometimes there isn't 1:1 mapping from dependency types to package managers -
+RPM, for example, is handled by YUM on Fedora, but there are different tools
+on other platforms, e.g. Zypper on OpenSuse. PackageManager subclasses should always
+represent these high-level tools like YUM or Zypper, not RPM itself.
 """
-TODO:
- * merge prompts
-  * issue only one prompt for dependencies
-  * and one prompt for polkit
- * figure out how to install using pip when 'venv' option is specified
- * write tests
-"""
+from __future__ import print_function
 import collections
-import platform
+import math
+import sys
 import tempfile
+import threading
 
+from devassistant import current_run
 from devassistant.command_helpers import ClHelper, DialogHelper
-
 from devassistant.logger import logger
 from devassistant import exceptions
+from devassistant import utils
+from devassistant import settings
 
-# list of shortcuts to managers
+# mapping of dependency types to managers that handle them
+# e.g. {'rpm': [YUMPackageManager, DNFPackageManager],
+#       'pip': [PIPPackageManager]}
 managers = {}
 
 def register_manager(manager):
-    managers[manager.shortcut] = manager
+    managers.setdefault(manager.shortcut, [])
+    managers[manager.shortcut].append(manager)
     return manager
 
 class PackageManager(object):
-    """ Abstract class for API definition of package managers """
+    """Abstract class for API definition of package managers."""
 
-    # Indicates whether this is a system manager. If so, dependencies of its type should be
-    # skipped on platforms where it is not the default system manager.
+    # Indicates whether this is a system manager.
     is_system = True
 
-    @classmethod
-    def match(cls, *args, **kwargs):
-        """
-        Return True if this package manager should be chosen as dep installer
-        """
-        raise NotImplementedError()
-
     @classmethod
     def get_perm_prompt(cls, package_list):
         """
@@ -58,17 +69,9 @@ class PackageManager(object):
         """
         raise NotImplementedError()
 
-    @classmethod
-    def is_installed(cls, *args, **kwargs):
-        """Is this manager available?"""
-        raise NotImplementedError()
-
     @classmethod
     def works(cls, *args, **kwargs):
-        """Raises exceptions.PackageManagerNotOperational if this manager
-        can't be used (something's missing) - e.g. for rpm manager
-        is_installed() returns whether rpm is installed, but works() finds
-        out whether the manager is usable - e.g. if we can install via yum."""
+        """Returns True if this package manager is usable, False otherwise."""
         raise NotImplementedError()
 
     @classmethod
@@ -100,9 +103,11 @@ class PackageManager(object):
 
 
 @register_manager
-class RPMPackageManager(PackageManager):
-    """ Package manager for managing rpm packages from repositories """
-    permission_prompt = "Installing {num} RPM package{plural}. Is this ok?"
+class YUMPackageManager(PackageManager):
+    """Package manager for managing rpm packages from repositories by Yum.
+    TODO: when we start using another RPM platform with another installer (OpenSuse),
+    we will need to pull out the RPM stuff into common superclass."""
+    permission_prompt = "Installing {num} RPM package{plural} by Yum. Is this ok?"
     shortcut = 'rpm'
 
     c_rpm = 'rpm'
@@ -151,10 +156,6 @@ class RPMPackageManager(PackageManager):
         logger.info('Installed {0}'.format(found_rpm), extra={'event_type': 'dep_installed'})
         return found_rpm
 
-    @classmethod
-    def match(cls, dep_t):
-        return dep_t == cls.shortcut
-
     @classmethod
     def install(cls, *args):
         cmd = ['pkexec', cls.c_yum, '-y', 'install']
@@ -166,23 +167,13 @@ class RPMPackageManager(PackageManager):
         except exceptions.ClException:
             return False
 
-    @classmethod
-    def is_installed(cls, dep):
-        try:
-            ClHelper('which rpm')
-            return True
-        except exceptions.ClException:
-            return False
-
     @classmethod
     def works(cls):
         try:
             import yum
             return True
         except ImportError:
-            msg = 'Cannot install RPM dependencies, YUM not importable.'
-            logger.error(msg)
-            raise exceptions.PackageManagerNotOperational(msg)
+            return False
 
     @classmethod
     def is_pkg_installed(cls, pkg):
@@ -190,8 +181,6 @@ class RPMPackageManager(PackageManager):
 
     @classmethod
     def resolve(cls, *args):
-        # TODO: we may need to rewrite this for e.g. suse, which
-        # is rpm based, but doesn't use yum; same for install()/is_available()/can_operate()
         logger.info('Resolving RPM dependencies ...')
         import yum
         y = yum.YumBase()
@@ -204,7 +193,6 @@ class RPMPackageManager(PackageManager):
                     y.install(y.returnPackageByDep(pkg))
                 except yum.Errors.YumBaseError:
                     msg = 'Package not found: {pkg}'.format(pkg=pkg)
-                    logger.error(msg)
                     raise exceptions.DependencyException(msg)
         y.resolveDeps()
         logger.debug('Installing/Updating:')
@@ -219,6 +207,68 @@ class RPMPackageManager(PackageManager):
         return "rpm package manager"
 
 
+@register_manager
+class PacmanPackageManager(PackageManager):
+    """Package manager for managing Arch Linux packages by pacman."""
+    permission_prompt = "Installing {num} package{plural} by Pacman. Is this ok?"
+    shortcut = 'pacman'
+
+    c_pacman = 'pacman'
+
+    @classmethod
+    def install(cls, *args):
+        cmd = ['pkexec', cls.c_pacman, '-S', '--noconfirm']
+        quoted_pkgs = map(lambda pkg: '"{pkg}"'.format(pkg=pkg), args)
+        cmd.extend(quoted_pkgs)
+        try:
+            ClHelper.run_command(' '.join(cmd), ignore_sigint=True)
+            return args
+        except exceptions.ClException:
+            return False
+
+    @classmethod
+    def is_pacmanpkg_installed(cls, pkg_name):
+        logger.info('Checking for presence of {0}...'.format(pkg_name), extra={'event_type': 'dep_check'})
+
+        try:
+            found_pkg = ClHelper.run_command('{pacman} -Q "{pkg}"'.\
+                                             format(pacman=cls.c_pacman, pkg=pkg_name))
+            logger.info('Found {0}'.format(found_pkg), extra={'event_type': 'dep_found'})
+            return found_pkg
+        except exceptions.ClException:
+            logger.info('Not found, will install', extra={'event_type': 'dep_not_found'})
+            return False
+
+    @classmethod
+    def is_group_installed(cls, group):
+        logger.info('Checking for presence of group {0}...'.format(group))
+
+        try:
+            output = ClHelper.run_command('{pacman} -Qg "{group}"'.\
+                                          format(pacman=cls.c_pacman,
+                                                 group=group))
+            return group
+        except exceptions.ClException:
+            return False
+
+    @classmethod
+    def works(cls):
+        try:
+            ClHelper.run_command('which pacman')
+            return True
+        except exceptions.ClException:
+            return False
+
+    @classmethod
+    def is_pkg_installed(cls, pkg):
+        return cls.is_pacmanpkg_installed(pkg) or cls.is_group_installed(pkg)
+
+    @classmethod
+    def resolve(cls, *args):
+        # TODO: I currently see no way how to just resolve dependencies by pacman
+        return args
+
+
 @register_manager
 class PIPPackageManager(PackageManager):
     """ Package manager for managing python dependencies from PyPI """
@@ -228,10 +278,6 @@ class PIPPackageManager(PackageManager):
 
     c_pip = 'pip'
 
-    @classmethod
-    def match(cls, dep_t):
-        return dep_t == cls.shortcut
-
     @classmethod
     def install(cls, *args):
         cmd = [cls.c_pip, 'install', '--user']
@@ -243,23 +289,13 @@ class PIPPackageManager(PackageManager):
         except exceptions.ClException:
             return False
 
-    @classmethod
-    def is_installed(cls):
-        try:
-            ClHelper.run_command('which pip')
-            return True
-        except exceptions.ClException:
-            return False
-
     @classmethod
     def works(cls):
         try:
             ClHelper.run_command('pip')
             return True
         except exceptions.ClException:
-            msg = '"pip" not found, can\'t install packages from PyPI.'
-            logger.error(msg)
-            raise exceptions.PackageManagerNotOperational(msg)
+            return False
 
     @classmethod
     def is_pkg_installed(cls, dep):
@@ -300,10 +336,6 @@ class NPMPackageManager(PackageManager):
 
     c_npm = 'npm'
 
-    @classmethod
-    def match(cls, dep_t):
-        return dep_t == cls.shortcut
-
     @classmethod
     def install(cls, *args):
         cmd = [cls.c_npm, 'install']
@@ -315,23 +347,13 @@ class NPMPackageManager(PackageManager):
         except exceptions.ClException:
             return False
 
-    @classmethod
-    def is_installed(cls):
-        try:
-            ClHelper.run_command('which npm')
-            return True
-        except exceptions.ClException:
-            return False
-
     @classmethod
     def works(cls):
         try:
             ClHelper.run_command('npm')
             return True
         except exceptions.ClException as e:
-            msg = 'Package manager for "{0}" not operational: {1}'.format(dep_t, e)
-            logger.error(msg)
-            raise exceptions.PackageManagerNotOperational(msg)
+            return False
 
     @classmethod
     def is_pkg_installed(cls, dep):
@@ -362,6 +384,19 @@ class NPMPackageManager(PackageManager):
 
 
 class DependencyInstaller(object):
+    """Installs all dependencies given to install() like this:
+    - Calls _process_dependency for each dependency type, system dependencies always go first
+      - "Implodes" the dependencies - e.g. if more 'rpm' lists were given, it creates one
+        list of all of them
+      - If it encounters system dependencies native for another system, it throws them away
+      - For non-system dependency type (e.g. 'gem', 'pip'), it also adds a system dependency
+        that has the ability to install these (e.g. rubygems, python-pip)
+    - Calls _install_dependencies
+      - For each dependency type
+        - Gets proper manager to install it
+        - Resolves dependencies of those dependencies :)
+        - Installs the dependencies
+    """
     # True if devassistant is installing dependencies and we can't interrupt the process
     install_lock = False
 
@@ -374,17 +409,20 @@ class DependencyInstaller(object):
 
     def get_package_manager(self, dep_t):
         """Choose proper package manager and return it."""
-        # one package manager can possibly handle multiple dep types,
-        # so we can't just do manager.shortcut == dep_t
-        for manager in managers.values():
-            if manager.match(dep_t):
+        mgrs = managers.get(dep_t, [])
+        for manager in mgrs:
+            if manager.works():
                 return manager
-        err = "Package manager for dependency type {0} was not found".format(dep_t)
-        logger.error(err)
-        raise exceptions.PackageManagerUnknown(err)
+        if not mgrs:
+            err = 'No package manager for dependency type "{dep_t}"'.format(dep_t=dep_t)
+            raise exceptions.NoPackageManagerException(err)
+        else:
+            err = 'No working package manager for "{dep_t}" in {mgrs}'.format(dep_t=dep_t,
+                                                                              mgrs=mgrs)
+            raise exceptions.PackageManagerOperationalException(err)
 
     def _process_dependency(self, dep_t, dep_l):
-        """Add depednecnies into self.dependencies, possibly also adding system packages
+        """Add dependencies into self.dependencies, possibly also adding system packages
         that contain non-distro package managers (e.g. if someone wants to install
         dependencines with pip and pip is not present, it will get installed through
         RPM on RPM based systems, etc.
@@ -392,17 +430,23 @@ class DependencyInstaller(object):
         Skips dependencies that are supposed to be installed by system manager that
         is not native to this system.
         """
-        if managers[dep_t].is_system and self.get_system_package_manager_shortcut() != dep_t:
+        if dep_t not in managers:
+            err = 'No package manager for dependency type "{dep_t}"'.format(dep_t=dep_t)
+            raise exceptions.NoPackageManagerException(err)
+        # try to get list of distros where the dependency type is system type
+        distro = settings.SYSTEM_DEPTYPES_SHORTCUTS.get(dep_t, None)
+        if not distro: # non-distro dependency type
+            sysdep_t = self.get_system_deptype_shortcut()
+            # for now, just take the first manager that can install dep_t and install this manager
+            self._process_dependency(sysdep_t,
+                                     managers[dep_t][0].get_distro_dependencies(sysdep_t))
+        elif utils.get_distro_name() not in distro: # distro dependency type, but for another distro
             return
-        if not managers[dep_t].is_system and not managers[dep_t].is_installed():
-            smgr_sc = self.get_system_package_manager_shortcut()
-            self._process_dependency(smgr_sc, managers[dep_t].get_distro_dependencies(smgr_sc))
         self.dependencies.setdefault(dep_t, [])
         self.dependencies[dep_t].extend(dep_l)
 
     def _ask_to_confirm(self, pac_man, *to_install):
         """ Return True if user wants to install packages, False otherwise """
-        message = '\n'.join(sorted(to_install))
         ret = DialogHelper.ask_for_package_list_confirm(
             prompt=pac_man.get_perm_prompt(to_install),
             package_list=to_install,
@@ -427,20 +471,31 @@ class DependencyInstaller(object):
             confirm = self._ask_to_confirm(pkg_mgr, *to_install)
             if not confirm:
                 msg = 'List of packages denied by user, exiting.'
-                logger.error(msg)
                 raise exceptions.DependencyException(msg)
 
             type(self).install_lock = True
-            logger.info('Installing dependencies, sit back and relax ...')
+            # TODO: we should do this more systematically (send signal to cl/gui?)
+            logger.info('Installing dependencies, sit back and relax ...',
+                        extra={'event_type': 'dep_installation_start'})
+            if current_run.UI == 'cli': # TODO: maybe let every manager to decide when to start
+                event = threading.Event()
+                t = FakeProgressThread(event)
+                t.start()
             installed = pkg_mgr.install(*to_install)
+            if current_run.UI == 'cli':
+                event.set()
+                t.join()
             type(self).install_lock = False
 
+            log_extra = {'event_type': 'dep_installation_end'}
             if not installed:
                 msg = 'Failed to install dependencies, exiting.'
-                logger.error(msg)
-                raise exceptions.DependencyException(msg)
+                logger.error(msg, extra=log_extra)
+                ex = exceptions.DependencyException(msg)
+                ex.already_logged = True
+                raise ex
             else:
-                logger.info('Successfully installed dependencies!')
+                logger.info('Successfully installed dependencies!', extra=log_extra)
 
     def install(self, struct):
         """
@@ -451,19 +506,36 @@ class DependencyInstaller(object):
         structure)
         """
         # the system dependencies should always go first
-        self.dependencies.setdefault(self.get_system_package_manager_shortcut(), [])
+        self.dependencies.setdefault(self.get_system_deptype_shortcut(), [])
         for dep_dict in struct:
             for dep_t, dep_l in dep_dict.items():
                 self._process_dependency(dep_t, dep_l)
         if self.dependencies:
             self._install_dependencies()
 
-    def get_system_package_manager_shortcut(self):
-        di = platform.linux_distribution()[0].lower()
-        # TODO: rpm by default, custom logic for other distros
+    def get_system_deptype_shortcut(self):
+        distro = utils.get_distro_name()
+        for k, v in settings.SYSTEM_DEPTYPES_SHORTCUTS.items():
+            if distro in v:
+                return k
 
+        # just try rpm if unkown (not very nice?)
         return 'rpm'
 
+class FakeProgressThread(threading.Thread):
+    def __init__(self, finish_event):
+        super(FakeProgressThread, self).__init__()
+        self.finish_event = finish_event
+
+    def run(self):
+        wait_for = 1
+        while not self.finish_event.isSet():
+            print('.', end='')
+            sys.stdout.flush()
+            wait_for += 1
+            self.finish_event.wait(math.log(wait_for))
+        print()
+
 
 def main():
     """ just for testing """
