@@ -11,6 +11,8 @@ LOG_FORMATS_MAP = {'log_cmd': u'{levelname}: {msg}',
                    'cmd_out': u'| {msg}',
                    'cmd_retcode': u'|> retcode: {msg}'}
 LOG_LEVELS_MAP = {'d': 'DEBUG', 'i': 'INFO', 'w': 'WARNING', 'e': 'ERROR', 'c': 'CRITICAL'}
+LAST_LR_VAR = 'LAST_LRES'
+LAST_R_VAR = 'LAST_RES'
 SUBASSISTANT_PREFIX = 'subassistant'
 SUBASSISTANT_N_STRING = 'subassistant_{0}'
 DEPS_ONLY_FLAG = '--deps-only'
@@ -18,4 +20,12 @@ CACHE_FILE = os.path.expanduser('~/.devassistant/.cache.yaml')
 DATA_DIRECTORIES = [os.path.join(os.path.dirname(__file__), 'data'),
                     '/usr/local/share/devassistant',
                     os.path.expanduser('~/.devassistant')]
-ASSISTANT_ROLES=['crt', 'mod', 'prep']
+if 'DEVASSISTANT_PATH' in os.environ:
+    DATA_DIRECTORIES = os.environ['DEVASSISTANT_PATH'].split(':') + DATA_DIRECTORIES
+ASSISTANT_ROLES=['crt', 'mod', 'prep', 'task']
+DEFAULT_ASSISTANT_ROLE = 'crt'
+
+# system dependency types and package managers for various distros
+# more distros can have the same system deptype, but different manager
+SYSTEM_DEPTYPES_SHORTCUTS = {'rpm': ['fedora'],
+                             'pacman': ['arch']}
