@@ -1,6 +1,7 @@
 import os
 
 from devassistant import cache
+from devassistant import current_run
 from devassistant.logger import logger
 from devassistant import yaml_loader
 from devassistant import settings
@@ -12,23 +13,24 @@ class YamlAssistantLoader(object):
     _assistants = {}
 
     @classmethod
-    def get_top_level_assistants(cls, roles=settings.ASSISTANT_ROLES):
-        """Returns list of top level assistants with specified roles (defaults to all roles).
+    def get_assistants(cls, superassistants):
+        """Returns list of assistants that are subassistants of given superassistants
+        (I love this docstring).
 
         Args:
             roles: list of names of roles, defaults to all roles
         Returns:
             list of YamlAssistant instances with specified roles
         """
-        cls.load_all_assistants(roles=roles)
+        cls.load_all_assistants(superassistants)
         result = []
-        for r in roles:
-            result.extend(cls._assistants[r])
+        for supa in superassistants:
+            result.extend(cls._assistants[supa.name])
 
         return result
 
     @classmethod
-    def load_all_assistants(cls, roles):
+    def load_all_assistants(cls, superassistants):
         """Fills self._assistants with loaded YamlAssistant instances of requested roles.
 
         Tries to use cache (updated/created if needed). If cache is unusable, it
@@ -37,22 +39,32 @@ class YamlAssistantLoader(object):
         Args:
             roles: list of required assistant roles
         """
-        to_load = set(roles) - set(cls._assistants.keys())
+        # {'crt': CreatorAssistant, ...}
+        superas_dict = dict(map(lambda a: (a.name, a), superassistants))
+        to_load = set(superas_dict.keys()) - set(cls._assistants.keys())
         for tl in to_load:
             dirs = [os.path.join(d, tl) for d in cls.assistants_dirs]
             file_hierarchy = cls.get_assistants_file_hierarchy(dirs)
-            try:
-                cch = cache.Cache()
-                cch.refresh_role(tl, file_hierarchy)
-                cls._assistants[tl] = cls.get_assistants_from_cache_hierarchy(cch.cache[tl],
-                                                                              role=tl)
-            except BaseException as e:
-                logger.debug(e)
+            # load all if we're not using cache or if we fail to load it
+            load_all = not current_run.USE_CACHE
+            if current_run.USE_CACHE:
+                try:
+                    cch = cache.Cache()
+                    cch.refresh_role(tl, file_hierarchy)
+                    cls._assistants[tl] = cls.get_assistants_from_cache_hierarchy(cch.cache[tl],
+                                                                                  superas_dict[tl],
+                                                                                  role=tl)
+                except BaseException as e:
+                    logger.debug(e)
+                    load_all = True
+            if load_all or not current_run.USE_CACHE:
                 cls._assistants[tl] = cls.get_assistants_from_file_hierarchy(file_hierarchy,
+                                                                             superas_dict[tl],
                                                                              role=tl)
 
     @classmethod
-    def get_assistants_from_cache_hierarchy(cls, cache_hierarchy, role='crt'):
+    def get_assistants_from_cache_hierarchy(cls, cache_hierarchy, superassistant,
+                                            role=settings.DEFAULT_ASSISTANT_ROLE):
         """Accepts cache_hierarch as described in devassistant.cache and returns
         instances of YamlAssistant (only with cached attributes) for loaded files
 
@@ -70,16 +82,19 @@ class YamlAssistantLoader(object):
         for name, attrs in cache_hierarchy.items():
             ass = cls.assistant_from_yaml(attrs['source'],
                                           {name: attrs['attrs']},
+                                          superassistant,
                                           fully_loaded=False,
                                           role=role)
             ass._subassistants = cls.get_assistants_from_cache_hierarchy(attrs['subhierarchy'],
+                                                                         ass,
                                                                          role=role)
             result.append(ass)
 
         return result
 
     @classmethod
-    def get_assistants_from_file_hierarchy(cls, file_hierarchy, role='crt'):
+    def get_assistants_from_file_hierarchy(cls, file_hierarchy, superassistant,
+                                           role=settings.DEFAULT_ASSISTANT_ROLE):
         """Accepts file_hierarch as returned by cls.get_assistant_file_hierarchy and returns
         instances of YamlAssistant for loaded files
 
@@ -92,13 +107,22 @@ class YamlAssistantLoader(object):
             references to instances of their subassistants (and their subassistants, ...)
         """
         result = []
+        warn_msg ='Failed to load assistant {source}, skipping subassistants.'
 
         for name, attrs in file_hierarchy.items():
             loaded_yaml = yaml_loader.YamlLoader.load_yaml_by_path(attrs['source'])
+            if not loaded_yaml: # there was an error parsing yaml
+                logger.warning(warn_msg.format(source=attrs['source']))
+                continue
             ass = cls.assistant_from_yaml(attrs['source'],
                                           loaded_yaml,
+                                          superassistant,
                                           role=role)
+            if not ass: # assistant isn't written properly (misses top-level mapping)
+                logger.warning(warn_msg.format(source=attrs['source']))
+                continue
             ass._subassistants = cls.get_assistants_from_file_hierarchy(attrs['subhierarchy'],
+                                                                        ass,
                                                                         role=role)
             result.append(ass)
 
@@ -138,21 +162,28 @@ class YamlAssistantLoader(object):
         return result
 
     @classmethod
-    def assistant_from_yaml(cls, source, y, fully_loaded=True, role='crt'):
+    def assistant_from_yaml(cls, source, y, superassistant, fully_loaded=True,
+                            role=settings.DEFAULT_ASSISTANT_ROLE):
         """Constructs instance of YamlAssistant loaded from given structure y, loaded
         from source file source.
 
         Args:
             source: path to assistant source file
             y: loaded yaml structure
+            superassistant: superassistant of this assistant
         Returns:
             YamlAssistant instance constructed from y with source file source
         """
-        # assume only one key and value
-        name, attrs = y.popitem()
+        try:
+            name, attrs = y.popitem() # assume only one key and value
+        except AttributeError:
+            logger.warning('File {source} is not formatted properly - no top level mapping.'.\
+                    format(source=source))
+            return None
         assistant = yaml_assistant.YamlAssistant(name,
                                                  attrs,
                                                  source,
+                                                 superassistant,
                                                  fully_loaded=fully_loaded,
                                                  role=role)
 
