@@ -3,7 +3,7 @@ import os
 from devassistant import exceptions
 from devassistant import yaml_loader
 from devassistant import settings
-from devassistant.assistants import snippet
+from devassistant import snippet
 
 class YamlSnippetLoader(object):
     snippets_dirs = list(map(lambda x: os.path.join(x, 'snippets'), settings.DATA_DIRECTORIES))
@@ -21,13 +21,12 @@ class YamlSnippetLoader(object):
         found = cls._find_snippet(name)
         if found != None:
             return found
-        loaded = yaml_loader.YamlLoader.load_yaml(cls.snippets_dirs, name)
+        loaded = yaml_loader.YamlLoader.load_yaml_by_relpath(cls.snippets_dirs, name + '.yaml')
         if loaded:
             path, parsed_yaml = loaded
             snip = snippet.Snippet(name,
                                    parsed_yaml,
-                                   path,
-                                   template_dir=yaml_loader.YamlLoader._default_template_dir_for(path))
+                                   path)
             cls._snippets[snip.path] = snip
             return snip
 
